//------------------------------------------------------------------
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_level1  0
#property indicator_level2  20
#property indicator_level3  80

//
//
//
//
//

extern int    StoPeriod                 = 14;
extern int    StoSlowing                =  6;
extern int    StoPrice                  =  0;
extern int    WprPeriod                 = 14;
extern int    MomPeriod                 = 14;
extern int    MomPrice                  = PRICE_CLOSE;
extern double MomModifier               =  0;
extern int    RsiPeriod                 = 14;
extern int    RsiPrice                  = PRICE_CLOSE;
extern int    CciPeriod                 = 14;
extern int    CciPrice                  = PRICE_TYPICAL;
extern int    UltFast                   =  7;
extern int    UltMiddle                 = 14;
extern int    UltSlow                   = 28;
extern int    UltFastK                  =  4;
extern int    UltMiddleK                =  2;
extern int    UltSlowK                  =  1;
extern int    MfiPeriod                 = 14;
extern int    arrowSize                 = 0; 
extern bool   drawDivergences           = true;
extern bool   ShowClassicalDivergence   = true;
extern bool   ShowHiddenDivergence      = true;
extern bool   drawIndicatorTrendLines   = true;
extern bool   drawPriceTrendLines       = true;
extern string drawLinesIdentificator    = "kubidiverge1";
extern double arrowsDisplacement        = 3.0;
extern bool   divergenceAlert           = true;
extern bool   divergenceAlertsMessage   = true;
extern bool   divergenceAlertsSound     = true;
extern bool   divergenceAlertsEmail     = false;
extern bool   divergenceAlertsNotify    = false;
extern string divergenceAlertsSoundName = "alert1.wav";

double value[];
double temp[];
double bullishDivergence[];
double bearishDivergence[];
string indicatorName;
string labelNames;

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,bullishDivergence); SetIndexStyle(0,DRAW_ARROW,0,arrowSize); SetIndexArrow(0,233);
   SetIndexBuffer(1,bearishDivergence); SetIndexStyle(1,DRAW_ARROW,0,arrowSize); SetIndexArrow(1,234);
   SetIndexBuffer(2,value);
   SetIndexBuffer(3,temp);
   labelNames    = "kubi_DivergenceLine "+drawLinesIdentificator+":";
   indicatorName = "kubi Divergence";
   IndicatorShortName(indicatorName);
      
   return(0);
}
int deinit()
{
   int length=StringLen(labelNames);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if(StringSubstr(name,0,length) == labelNames)  ObjectDelete(name);   
   }
   return(0);
}


//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
   double divider = UltFastK+UltMiddleK+UltSlowK;
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--) temp[i] = Close[i]-MathMin(Low[i],Close[i+1]);
   for(i=limit; i>=0; i--)
   {
      double atrf = iATR(NULL,0,UltFast  ,i);
      double atrm = iATR(NULL,0,UltMiddle,i);
      double atrs = iATR(NULL,0,UltSlow  ,i);
      double ult  = 0;
      if (atrf!=0 && atrm !=0 && atrs !=0)
             ult = (UltFastK  *iMAOnArray(temp,0,UltFast  ,0,MODE_SMA,i)/atrf +
                    UltMiddleK*iMAOnArray(temp,0,UltMiddle,0,MODE_SMA,i)/atrm +
                    UltSlowK  *iMAOnArray(temp,0,UltSlow  ,0,MODE_SMA,i)/atrs)/divider*100;
                    
            //
            //
            //
            //
            //
                                
                                
            value[i] = (iStochastic(NULL,0,StoPeriod,1,StoSlowing,MODE_SMA,StoPrice,MODE_MAIN,i)+
                        iWPR(NULL,0,WprPeriod,i)+
                        iMomentum(NULL,0,MomPeriod,MomPrice,i)+MomModifier+
                        iRSI(NULL,0,RsiPeriod,RsiPrice,i)+
                        iCCI(NULL,0,CciPeriod,CciPrice,i)+
                        iMFI(NULL,0,MfiPeriod,i)+ult)/7.0;
                        if (drawDivergences)
                        {
                          CatchBullishDivergence(i);
                          CatchBearishDivergence(i);
                        }           
   }
   return(0);
}

//
//
//
//
//

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  

   //
   //
   //
   //
   //
      
   int currentLow = shift;
   int lastLow    = GetIndicatorLastLow(shift+1);
   if (value[currentLow] > value[lastLow] && Low[currentLow] < Low[lastLow])
   {
     if (ShowClassicalDivergence)
     {
        bullishDivergence[currentLow] = value[currentLow] - arrowsDisplacement;
        if (drawPriceTrendLines)    DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],Green,STYLE_SOLID);
        if (drawIndicatorTrendLines)DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],value[currentLow],value[lastLow],Green,STYLE_SOLID);
        if (divergenceAlert)        DisplayAlert("Classical bullish divergence",currentLow);  
     }
                        
   }
     
   //
   //
   //
   //
   //
        
   if (value[currentLow] < value[lastLow] && Low[currentLow] > Low[lastLow])
   {
     if (ShowHiddenDivergence)
     {
        bullishDivergence[currentLow] = value[currentLow] - arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],Green, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],value[currentLow], value[lastLow],Green, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert("Reverse bullish divergence",currentLow); 
     }
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(IsIndicatorPeak(shift) == false) return;
   int currentPeak = shift;
   int lastPeak = GetIndicatorLastPeak(shift+1);

   //
   //
   //
   //
   //
      
   if (value[currentPeak] < value[lastPeak] && High[currentPeak]>High[lastPeak])
   {
     if (ShowClassicalDivergence)
     {
        bearishDivergence[currentPeak] = value[currentPeak] + arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],Red,STYLE_SOLID);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],value[currentPeak],value[lastPeak],Red,STYLE_SOLID);
        if (divergenceAlert)         DisplayAlert("Classical bearish divergence",currentPeak);
     } 
                        
   }
   
   //
   //
   //
   //
   //

   if (value[currentPeak] > value[lastPeak] && High[currentPeak] < High[lastPeak])
   {
     if (ShowHiddenDivergence)
     {
        bearishDivergence[currentPeak] = value[currentPeak] + arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],       Red, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],value[currentPeak],value[lastPeak], Red, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert("Reverse bearish divergence",currentPeak);
     }
                         
   }   
}

//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(value[shift] >= value[shift+1] && value[shift] > value[shift+2] && value[shift] > value[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool IsIndicatorLow(int shift)
{
   if(value[shift] <= value[shift+1] && value[shift] < value[shift+2] && value[shift] < value[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastPeak(int shift)
{
    for(int i = shift+5; i < Bars; i++)
    {
       if(value[i] >= value[i+1] && value[i] > value[i+2] && value[i] >= value[i-1] && value[i] > value[i-2])
         return(i);
    }
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastLow(int shift)
{
    for (int i = shift+5; i < Bars; i++)
    {
       if (value[i] <= value[i+1] && value[i] < value[i+2] && value[i] <= value[i-1] && value[i] < value[i-2])
         return(i);
}
     
return(-1);
}

//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," kubi ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," kubi " +" "+dmessage));
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," kubi "),dmessage);
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}


//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    string label = labelNames+first+"os"+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    int indicatorWindow = WindowFind(indicatorName);
    if (indicatorWindow < 0) return;
    
    //
    //
    //
    //
    //
    
    string label = labelNames+first+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}